#include "Planet.h"
#include "Constants.h"
#include "Colors.h"
#include <cassert>
#include <stdexcept>

Planet::Planet()
	:
	action_list(1250, 1550, 70, 750)
{
	project = Projects::start;
	loc = EMPTY;
	colour = Colors::Black;
}

Planet::Planet (Location location,std::mt19937& rng)
	:
	action_list(1250,1550,70,750)
{
	std::uniform_int_distribution<int> d2(1, 2);
	std::uniform_int_distribution<int> d3(1, 3);
	std::uniform_int_distribution<int> d6(1, 6);
	loc = location;
	project = Projects::start;
	int habitable = d2(rng);
	for (int i = 0; i < 9; i++)
	{
		if (habitable == 1)
		{
			if (d2(rng) == 1)
				map[i].yield = 0;
			else if (d2(rng) == 1)
				map[i].yield = 1;
			else if (d3(rng) == 1)
				map[i].yield = 3;
			else
				map[i].yield = 2;
		}
		map[i].improvement = ' ';
		if (d6(rng)==1)
		{
			map[i].zest = true;
		}
	}

	int total_yield = 0;
	for (int i = 0; i < 9; i++)
		total_yield += map[i].yield;

	if (total_yield >= 20)
	{
		type = 7;
		colour = Colors::MakeRGB(0u, 51u, 0u);
	}
	else if (total_yield >= 15)
	{
		type = 6;
		colour = Colors::MakeRGB(0u, 128u, 0u);
	}
	else if (total_yield >= 12)
	{
		type = 5;
		colour = Colors::MakeRGB(51u, 51u, 0u);
	}
	else if (total_yield >= 9)
	{
		type = 4;
		colour = Colors::MakeRGB(128u, 128u, 0u);
	}
	else if (total_yield >= 6)
	{
		type = 3;
		colour = Colors::MakeRGB(153u, 204u, 0u);
	}
	else if (total_yield >= 3)
	{
		type = 2;
		colour = Colors::MakeRGB(255u, 204u, 0u);
	}
	else if (total_yield >= 1)
	{
		type = 1;
		colour = Colors::MakeRGB(255u, 153u, 0u);
	}
}

int Planet::get_population()
{
	int count = 0;
	for (int i = 0; i < 9; i++)
	{
		count += map[i].person.size();
	}
	return count;
}

int Planet::get_engineering()
{
	int engineering = 0;
	for (int i = 0; i < 9; i++)
	{
		for (auto& j:map[i].person)
		{
			if (j.job == 'e')
				engineering += j.ability;
		}
	}
	return engineering;
}

int Planet::get_defense()
{
	int defense = 0;
	for (int i = 0; i < 9; i++)
	{
		for (auto& j:map[i].person)
		{
			if (j.job == 's')
				defense += j.ability;
		}
	}
	return defense;
}

void Planet::set_loc(Location newloc)
{
	loc.x = newloc.x;
	loc.y = newloc.y;
}

Location Planet::get_loc()
{
	return loc;
}

tile Planet::get_tile(int i)
{
	return map[i];
}

int Planet::get_tile_yield(int i)
{
	return map[i].yield;
}

void Planet::print(Graphics& gfx,Font& f)
{
	for (int i = 0; i < 9; i++)
	{
		Location newloc;
		newloc.x = GALAXY_WIDTH * SECTOR_DIMENSION + (i % 3) * 100;
		newloc.y = (i / 3) * 100;
		if (map[i].yield == 0)
		{
			gfx.DrawRectDim(newloc.x,newloc.y,100,100, Colors::MakeRGB(255u,153u,0u));
		}
		else if (map[i].yield == 1)
		{
			gfx.DrawRectDim(newloc.x, newloc.y, 100, 100, Colors::MakeRGB(128u,128u,0u));
		}
		else if (map[i].yield == 2)
		{
			gfx.DrawRectDim(newloc.x, newloc.y, 100, 100, Colors::MakeRGB(0u,128u,0u));
		}
		else if (map[i].yield == 3)
		{
			gfx.DrawRectDim(newloc.x, newloc.y, 100, 100, Colors::MakeRGB(0u,51u,0u));
		}
		if (map[i].zest == true)
		{
			Color c = Colors::Black;
			if (map[i].yield == 3)
				c = Colors::White;
			std::string zest = "Z";
			f.drawText(zest, { newloc.x + 42,newloc.y + 36 }, gfx, c);
		}
	}
}

void Planet::print(Location loc1,Graphics& gfx,Font& f)
{
	for (int i = 0; i < 9; i++)
	{
		Location newloc;
		newloc.x = GALAXY_WIDTH * SECTOR_DIMENSION + (i % 3) * 100 + loc1.x;
		newloc.y = (i / 3) * 100 + loc1.y;
		if (map[i].yield == 0)
		{
			gfx.DrawRectDim(newloc.x, newloc.y, 100, 100, Colors::MakeRGB(255u, 153u, 0u));
		}
		else if (map[i].yield == 1)
		{
			gfx.DrawRectDim(newloc.x, newloc.y, 100, 100, Colors::MakeRGB(128u, 128u, 0u));
		}
		else if (map[i].yield == 2)
		{
			gfx.DrawRectDim(newloc.x, newloc.y, 100, 100, Colors::MakeRGB(0u, 128u, 0u));
		}
		else if (map[i].yield == 3)
		{
			gfx.DrawRectDim(newloc.x, newloc.y, 100, 100, Colors::MakeRGB(0u, 51u, 0u));
		}
		if (map[i].zest == true)
		{
			Color c = Colors::Black;
			if (map[i].yield == 3)
				c = Colors::White;
			std::string zest = "Z";
			f.drawText(zest, { newloc.x + 42,newloc.y + 36 }, gfx, c);
		}
	}
}

void Planet::print_colony(Graphics& gfx, Font& f)
{
	for (int i = 0; i < 9; i++)
	{
		Location newloc;
		newloc.x = GALAXY_WIDTH * SECTOR_DIMENSION + (i % 3) * 100;
		newloc.y = (i / 3) * 100;
		if (map[i].yield == 0)
		{
			gfx.DrawRectDim(newloc.x, newloc.y, 100, 100, Colors::MakeRGB(255u, 153u, 0u));
		}
		else if (map[i].yield == 1)
		{
			gfx.DrawRectDim(newloc.x, newloc.y, 100, 100, Colors::MakeRGB(128u, 128u, 0u));
		}
		else if (map[i].yield == 2)
		{
			gfx.DrawRectDim(newloc.x, newloc.y, 100, 100, Colors::MakeRGB(0u, 128u, 0u));
		}
		else if (map[i].yield == 3)
		{
			gfx.DrawRectDim(newloc.x, newloc.y, 100, 100, Colors::MakeRGB(0u, 51u, 0u));
		}
		std::string c(1, map[i].improvement);
		std::string zest = "Z";
		Color zest_colour = Colors::MakeRGB(255u, 0u, 255u);
		Color imp_colour = Colors::Black;
		if (map[i].yield == 3)
			imp_colour = Colors::White;
		if (map[i].zest == false && map[i].improvement != ' ')
		{
			f.drawText(c, { newloc.x + 42,newloc.y + 36 }, gfx, imp_colour);
		}
		else if (map[i].zest == true && map[i].improvement != ' ')
		{
			f.drawText(c, { newloc.x + 23,newloc.y + 36 }, gfx, imp_colour);
			f.drawText(zest, { newloc.x + 61,newloc.y + 36 }, gfx, zest_colour);
		}
		else if (map[i].zest == true && map[i].improvement == ' ')
		{
			f.drawText(zest, { newloc.x + 42,newloc.y + 36 }, gfx, zest_colour);
		}
	}

}

void Planet::print_colony(Location loc1,Graphics& gfx, Font& f)
{
	for (int i = 0; i < 9; i++)
	{
		Location newloc;
		newloc.x = GALAXY_WIDTH * SECTOR_DIMENSION + (i % 3) * 100 + loc1.x;
		newloc.y = (i / 3) * 100 + loc1.y;
		if (map[i].yield == 0)
		{
			gfx.DrawRectDim(newloc.x, newloc.y, 100, 100, Colors::MakeRGB(255u, 153u, 0u));
		}
		else if (map[i].yield == 1)
		{
			gfx.DrawRectDim(newloc.x, newloc.y, 100, 100, Colors::MakeRGB(128u, 128u, 0u));
		}
		else if (map[i].yield == 2)
		{
			gfx.DrawRectDim(newloc.x, newloc.y, 100, 100, Colors::MakeRGB(0u, 128u, 0u));
		}
		else if (map[i].yield == 3)
		{
			gfx.DrawRectDim(newloc.x, newloc.y, 100, 100, Colors::MakeRGB(0u, 51u, 0u));
		}
		std::string c(1, map[i].improvement);
		std::string zest = "Z";
		Color zest_colour = Colors::MakeRGB(255u, 0u, 255u);
		Color imp_colour = Colors::Black;
		if (map[i].yield == 3)
			imp_colour = Colors::White;
		if (map[i].zest == false && map[i].improvement != ' ')
		{
			f.drawText(c, { newloc.x + 42,newloc.y + 36 }, gfx, imp_colour);
		}
		else if (map[i].zest == true && map[i].improvement != ' ')
		{
			f.drawText(c, { newloc.x + 23,newloc.y + 36 }, gfx, imp_colour);
			f.drawText(zest, { newloc.x + 61,newloc.y + 36 }, gfx, zest_colour);
		}
		else if (map[i].zest == true && map[i].improvement == ' ')
		{
			f.drawText(zest, { newloc.x + 42,newloc.y + 36 }, gfx, zest_colour);
		}
	}

}

void Planet::build_improvement(int i, char c)
{
	map[i].improvement = c;
}

bool Planet::is_ontop(int i)
{
	if (map[i].improvement != ' ')
		return true;
	return false;
}

bool Planet::is_beside(int i)
{
	if (i == 0)
	{
		if (map[1].improvement != ' ')
			return true;
		if (map[3].improvement != ' ')
			return true;
	}
	if (i == 1)
	{
		if (map[0].improvement != ' ')
			return true;
		if (map[2].improvement != ' ')
			return true;
		if (map[4].improvement != ' ')
			return true;
	}
	if (i == 2)
	{
		if (map[1].improvement != ' ')
			return true;
		if (map[5].improvement != ' ')
			return true;
	}
	if (i == 3)
	{
		if (map[0].improvement != ' ')
			return true;
		if (map[4].improvement != ' ')
			return true;
		if (map[6].improvement != ' ')
			return true;
	}
	if (i == 4)
	{
		if (map[1].improvement != ' ')
			return true;
		if (map[3].improvement != ' ')
			return true;
		if (map[5].improvement != ' ')
			return true;
		if (map[7].improvement != ' ')
			return true;
	}
	if (i == 5)
	{
		if (map[2].improvement != ' ' || map[8].improvement != ' ' || map[4].improvement != ' ')
			return true;
	}
	if (i == 6)
	{
		if (map[3].improvement != ' ' || map[7].improvement != ' ')
			return true;
	}
	if (i == 7)
	{
		if (map[4].improvement != ' ' || map[6].improvement != ' ' || map[8].improvement != ' ')
			return true;
	}
	if (i == 8)
	{
		if (map[5].improvement != ' ' || map[7].improvement != ' ')
			return true;
	}
	return false;
}

void Planet::add_person(int i, int id, std::mt19937& rng)
{
	assert(i >= 0);
	assert(i < 9);
	std::uniform_int_distribution<int> d6(1, 6);
	Person person;
	person.job = ' ';
	person.number = id;
	person.reeducations = 0;
	person.ability = d6(rng);
	person.active = true;
	map[i].person.push_back(person);
}

bool Planet::is_person(int id)
{
	for (int i = 0; i < 9; i++)
	{
		for (auto& p:map[i].person)
		{
			if (p.number == id)
				return true;
		}
	}
	return false;
}

Person Planet::get_person(int id)
{
	assert(is_person(id));
	for (int i = 0; i < 9; i++)
	{
		for (auto& p:map[i].person)
		{
			if (p.number == id)
				return p;
		}
	}
	throw std::runtime_error("no such person");
	Person o{};
	return o;
}

void Planet::set_job(int id, char j)
{
	for (int i = 0; i < 9; i++)
	{
		for (std::vector<Person>::reverse_iterator p = map[i].person.rbegin(); p != map[i].person.rend(); ++p)
		{
			if (p->number == id)
				p->job = j;
		}
	}
}

void Planet::set_project(Project p)
{
	project = p;
}

void Planet::add_action(std::string t, bool n, int num)
{
	Action a(t, n, num);
	action.push_back(a);
	Color c = Colors::White;
	if (n == true)
		c = Colors::Red;
	action_list.add_button(t, c,a);
}

void Planet::remove_action(int i)
{
	assert(i != 0);
	assert(i < action.size());
	if (i == (action.size()-1))
	{
		action.pop_back();
	}
	if (i < (action.size()-1))
	{
		action.erase(action.begin() + i);
	}
}

Color Planet::get_colour()
{
	return colour;
}

int Planet::get_food()
{
	return food;
}

int Planet::get_food_output()
{
	std::vector<int> farmer;
	std::vector<int> farm;
	int food_output = 0;
	int num_farmers = 0;
	int num_farms = 0;
	int stop = 0;
	for (int i = 0; i < 9; i++)
	{
		if (map[i].improvement == 'F')
		{
			farm.push_back(map[i].yield);
			num_farms++;
		}
		for (auto& j:map[i].person)
		{
			if (j.job == 'f')
			{
				farmer.push_back(j.ability);
				num_farmers++;
			}
		}
	}
	std::sort(farmer.begin(), farmer.begin() + num_farmers);
	std::sort(farm.begin(), farm.begin() + num_farms);
	if (num_farms > num_farmers)
		stop = num_farmers;
	else
		stop = num_farms;
	for (int i = 0; i < stop; i++)
	{
		food_output += farm[i] * farmer[i];
	}
	return food_output;
}

void Planet::update_food()
{
	food+= get_food_output() - get_population();
	if (food < 0)
		add_action("Starvation", true, abs(food));
}

void Planet::update_production()
{
	production_points += get_engineering();
	if (production_points >= project.cost)
	{
		production_points -= project.cost;
		add_action("Choose project", true,1);
		if (project.type == 'F')
			add_action("Place Farm", true, 1);
		else if (project.type == 'C')
			add_action("Place City", true, 1);
		else if (project.type == 'm')
			add_action("Place Mine", true, 1);
		else if (project.type == 'M')
			add_action("Place Megacity", true, 1);
		else if (project.type == 'L')
			add_action("Place Lab", true, 1);
		else if (project.type == 'W')
			add_action("Place Workplant", true, 1);
		else if (project.type == 'S')
			add_action("Place School", true, 1);
		else if (project.type == 'B')
			add_action("Place Barracks", true, 1);
		else if (project.type == 's')
			add_action("Place Ship", true,1);
		else if (project.type == 'w')
			warp_engines++;
		else if (project.type == 'a')
			antimatter_bombs++;
		else if (project.type == 'n')
			nuclear_missiles++;
	}
}

bool Planet::is_teacher()
{
	for (int i = 0; i < 9; i++)
	{
		for (auto& p:map[i].person)
		{
			if (p.job == 't' && p.active == true)
				return true;
		}
	}
	return false;
}

int Planet::num_active_teachers()
{
	int count = 0;
	for (int i = 0; i < 9; i++)
	{
		for (auto& p:map[i].person)
		{
			if (p.job == 't' && p.active == true)
				count++;
		}
	}
	return count;
}

int Planet::num_building(char type)
{
	int count = 0;
	for (int i = 0; i < 9; i++)
	{
		if (map[i].improvement == type)
			count++;
	}
	return count;
}

void Planet::check_reeducate()
{
	int count = 0;
	int num = 0;
	for (int i = 0; i < 9; i++)
	{
		for (auto& p:map[i].person)
		{
			if (p.reeducations > num_building('S'))
				count++;
		}
	}
	if (count > num_active_teachers())
		num = num_active_teachers();
	else
		num = count;
	if (num > 0)
	{
		add_action("Re-educate", false, num);
	}
}

int Planet::num_miners()
{
	int count = 0;
	for (int i = 0; i < 9; i++)
	{
		for (auto& p:map[i].person)
		{
			if (p.job == 'm')
				count++;
		}
	}
	return count;
}

void Planet::check_mining()
{
	int num = 0;
	if (num_miners() > num_building('m'))
		num = num_building('m');
	else
		num = num_miners();
	if (num > 0)
		add_action("Mine", false, num);
}

void Planet::set_up_food()
{
	food += get_food_output();
}

void Planet::set_colour(Color c)
{
	colour = c;
}

void Planet::print_info(Graphics& gfx,Font& f)
{
	Location loc = { 1250,335 };
	std::string text = "Production: ";
	f.drawText(text, loc, gfx, Colors::White);
	text = std::to_string(production_points);
	loc.x += 16 * 12;
	f.drawText(text, loc, gfx, Colors::White);
	text = "Engineering: ";
	loc.x = 1250;
	loc.y += 35;
	f.drawText(text, loc, gfx, Colors::White);
	text = std::to_string(get_engineering());
	loc.x += 16 * 13;
	f.drawText(text, loc, gfx, Colors::White);
	text = "Project: ";
	loc.x = 1250;
	loc.y += 35;
	f.drawText(text, loc, gfx, Colors::White);
	text = project.name;
	loc.x += 16 * 9;
	f.drawText(text, loc, gfx, Colors::White);
	text = "Built in ";
	loc.x = 1250;
	loc.y += 35;
	f.drawText(text, loc, gfx, Colors::White);
	text = std::to_string((int)ceil((double)(project.cost - production_points) / (double)get_engineering()));
	if ((int)ceil((double)(project.cost - production_points) / (double)get_engineering()) < 0)
		text = "0";
	loc.x += 16 * 9;
	f.drawText(text, loc, gfx, Colors::White);
	loc.x += text.size() * 16;
	if ((int)ceil((double)(project.cost - production_points) / (double)get_engineering()) == 1)
		text = " turn";
	else
		text = " turns";
	f.drawText(text, loc, gfx, Colors::White);
	text = "Food: ";
	loc.x = 1250;
	loc.y += 35;
	f.drawText(text, loc, gfx, Colors::White);
	text = std::to_string(food);
	loc.x += 16 * 6;
	f.drawText(text, loc, gfx, Colors::White);
	text = "Farming: ";
	loc.x = 1250;
	loc.y += 35;
	f.drawText(text, loc, gfx, Colors::White);
	text = std::to_string(get_food_output());
	loc.x += 16 * 9;
	f.drawText(text, loc, gfx, Colors::White);
	text = "Population: ";
	loc.x = 1250;
	loc.y += 35;
	f.drawText(text, loc, gfx, Colors::White);
	text = std::to_string(get_population());
	loc.x += 16 * 12;
	f.drawText(text, loc, gfx, Colors::White);
	text = "D:";
	loc.x = 1250;
	loc.y += 35;
	f.drawText(text, loc, gfx, Colors::White);
	text = std::to_string(dark_matter);
	loc.x += 16 * 2;
	f.drawText(text, loc, gfx, Colors::White);
	loc.x += 16 * text.size();
	text = " A:";
	f.drawText(text, loc, gfx, Colors::White);
	loc.x += 16 * text.size();
	text = std::to_string(antimatter);
	f.drawText(text, loc, gfx, Colors::White);
	loc.x += 16 * text.size();
	text = " U:";
	f.drawText(text, loc, gfx, Colors::White);
	loc.x += 16 * text.size();
	text = std::to_string(uranium);
	f.drawText(text, loc, gfx, Colors::White);
	loc.x = 1250;
	loc.y += 35;
	text = "E:";
	f.drawText(text, loc, gfx, Colors::White);
	loc.x += 16 * text.size();
	text = std::to_string(warp_engines);
	f.drawText(text, loc, gfx, Colors::White);
	loc.x += 16 * text.size();
	text = " B:";
	f.drawText(text, loc, gfx, Colors::White);
	loc.x += 16 * text.size();
	text = std::to_string(antimatter_bombs);
	f.drawText(text, loc, gfx, Colors::White);
	loc.x += 16 * text.size();
	text = " M:";
	f.drawText(text, loc, gfx, Colors::White);
	loc.x += 16 * text.size();
	text = std::to_string(nuclear_missiles);
	f.drawText(text, loc, gfx, Colors::White);
}

Project Planet::get_project()
{
	return project;
}

void Planet::remove_action(std::string t)
{
	std::size_t count = 0;
	for (const auto& i : action_list.button)
	{
		if (i.action.type == t)
		{
			if (count == (action_list.button.size() - 1))
				action_list.button.pop_back();
			else
				action_list.button.erase(action_list.button.begin() + count);
		}
		count++;
	}
}

int Planet::get_type()
{
	return type;
}