#pragma once
#include "Planet.h"
#include "Ship.h"
#include "Colors.h"
#include <string>
#include "Constants.h"
#include <random>
#include "Galaxy.h"

struct TileNode
{
	Location loc;
	TileNode* link;
};

class Player
{
public:
	Player();
	void add_colony(Planet p);
	void add_discovered(int i);
	void add_in_range(int i);
	void set_colour(Color c);
	Color get_colour();
	bool is_colony(Location loc);
	Planet get_colony(int i);
	bool is_discovered(int i);
	bool is_in_range(int i);
	void add_discovered_planet(Planet p);
	bool is_discovered_planet(Location loc);
	Planet get_planet(Location loc);
	void build_improvement(Location loc, int j, char c);
	void add_person(Location loc, int j, std::mt19937& rng);
	int get_idnum();
	void increment_idnum();
	Person get_person(int id);
	void set_job(int id, char j);
	void set_project(Location loc, Project p);
	const std::vector<Planet>::reverse_iterator& get_colony_rbegin();
	const std::vector<Planet>::reverse_iterator& get_colony_rend();
	const std::vector<Ship>::reverse_iterator& get_ship_rbegin();
	const std::vector<Ship>::reverse_iterator& get_ship_rend();
	void births(std::mt19937& rng);
	int get_tech(int i);
	void set_up_food();
	void update_discovered(Galaxy& g);
	void print(Graphics& gfx, Font& f, Galaxy& g);
public:
	bool set_up = false;
	bool first_city = false;
	bool first_farm = false;
	bool person1 = false;
	bool person2 = false;
	bool person3 = false;
	bool start_project = false;
	std::vector<Planet> colony;
	std::vector<Ship> ship;
	std::vector<bool> discovered;
	std::vector<bool> in_range;
	std::vector<Planet> colonizing_planet;
	std::vector<Planet> discovered_planet;
private:
	Color colour;
	int idnum = 1;
	int tech[10];
	// 0 = WARP DRIVE, 1 = MEGACITIES, 2 = ANTIMATTER BOMBS,
	// 3 = ADVANCED AGRICULTURE, 4 = GENE MANIPULATION,
	// 5 = LIFE SUPPORT, 6 = TERRAFORMING, 7 = NUCLEAR MISSILES
	// 8 = OPTICS, 9 = TELEPORTATION
};